@@ -69,6 +69,7 @@ type UnidlingController struct {
 	scaleNamespacer     scale.ScalesGetter
 	mapper              meta.RESTMapper
 	endpointsNamespacer corev1client.EndpointsGetter
+	servicesNamespacer  corev1client.ServicesGetter
 	queue               workqueue.RateLimitingInterface
 	lastFiredCache      *lastFiredCache
 
@@ -77,7 +78,7 @@ type UnidlingController struct {
 	rcNamespacer corev1client.ReplicationControllersGetter
 }
 
-func NewUnidlingController(scaleNS scale.ScalesGetter, mapper meta.RESTMapper, endptsNS corev1client.EndpointsGetter, evtNS corev1client.EventsGetter,
+func NewUnidlingController(scaleNS scale.ScalesGetter, mapper meta.RESTMapper, endptsNS corev1client.EndpointsGetter, servicesNS corev1client.ServicesGetter, evtNS corev1client.EventsGetter,
 	dcNamespacer appstypedclient.DeploymentConfigsGetter, rcNamespacer corev1client.ReplicationControllersGetter,
 	resyncPeriod time.Duration) *UnidlingController {
 	fieldSet := fields.Set{}
@@ -88,6 +89,7 @@ func NewUnidlingController(scaleNS scale.ScalesGetter, mapper meta.RESTMapper, e
 		scaleNamespacer:     scaleNS,
 		mapper:              mapper,
 		endpointsNamespacer: endptsNS,
+		servicesNamespacer:  servicesNS,
 		queue:               workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "unidling"),
 		lastFiredCache: &lastFiredCache{
 			items: make(map[types.NamespacedName]time.Time),
@@ -257,20 +259,20 @@ func (c *UnidlingController) awaitRequest() bool {
 }
 
 // handleRequest handles a single request to unidle.  After checking the validity of the request,
-// it will examine the endpoints in question to determine which scalables to scale, and will scale
-// them and remove them from the endpoints' list of idled scalables.  If it is unable to properly
+// it will examine the service in question to determine which scalables to scale, and will scale
+// them and remove them from the services' list of idled scalables.  If it is unable to properly
 // process the request, it will return a boolean indicating whether or not we should retry later,
 // as well as an error (e.g. if we're unable to parse an annotation, retrying later won't help,
 // so it will return false).
 func (c *UnidlingController) handleRequest(info types.NamespacedName, lastFired time.Time) (bool, error) {
-	// fetch the endpoints associated with the service in question
-	targetEndpoints, err := c.endpointsNamespacer.Endpoints(info.Namespace).Get(context.TODO(), info.Name, metav1.GetOptions{})
+	// fetch the service in question
+	targetService, err := c.servicesNamespacer.Services(info.Namespace).Get(context.TODO(), info.Name, metav1.GetOptions{})
 	if err != nil {
-		return true, fmt.Errorf("unable to retrieve endpoints: %v", err)
+		return true, fmt.Errorf("unable to retrieve service: %v", err)
 	}
 
 	// make sure we actually were idled...
-	idledTimeRaw, wasIdled := targetEndpoints.Annotations[unidlingapi.IdledAtAnnotation]
+	idledTimeRaw, wasIdled := targetService.Annotations[unidlingapi.IdledAtAnnotation]
 	if !wasIdled {
 		klog.V(5).Infof("UnidlingController received a NeedPods event for a service that was not idled, ignoring")
 		return false, nil
@@ -289,7 +291,7 @@ func (c *UnidlingController) handleRequest(info types.NamespacedName, lastFired
 
 	// TODO: ew, this is metav1.  Such is life when working with annotations.
 	var targetScalables []unidlingapi.RecordedScaleReference
-	if targetScalablesStr, hasTargetScalables := targetEndpoints.Annotations[unidlingapi.UnidleTargetAnnotation]; hasTargetScalables {
+	if targetScalablesStr, hasTargetScalables := targetService.Annotations[unidlingapi.UnidleTargetAnnotation]; hasTargetScalables {
 		if err = json.Unmarshal([]byte(targetScalablesStr), &targetScalables); err != nil {
 			// retrying here won't help, we're just stuck as idled since we can't parse the idled scalables list
 			return false, fmt.Errorf("unable to unmarshal target scalable references: %v", err)
@@ -355,25 +357,65 @@ func (c *UnidlingController) handleRequest(info types.NamespacedName, lastFired
 	}
 
 	if len(newAnnotationList) == 0 {
-		delete(targetEndpoints.Annotations, unidlingapi.UnidleTargetAnnotation)
-		delete(targetEndpoints.Annotations, unidlingapi.IdledAtAnnotation)
+		delete(targetService.Annotations, unidlingapi.UnidleTargetAnnotation)
+		delete(targetService.Annotations, unidlingapi.IdledAtAnnotation)
 	} else {
 		var newAnnotationBytes []byte
 		newAnnotationBytes, err = json.Marshal(newAnnotationList)
 		if err != nil {
 			utilruntime.HandleError(fmt.Errorf("unable to update/remove idle annotations from %s/%s: unable to marshal list of remaining scalables, removing list entirely: %v", info.Namespace, info.Name, err))
 
-			delete(targetEndpoints.Annotations, unidlingapi.UnidleTargetAnnotation)
-			delete(targetEndpoints.Annotations, unidlingapi.IdledAtAnnotation)
+			delete(targetService.Annotations, unidlingapi.UnidleTargetAnnotation)
+			delete(targetService.Annotations, unidlingapi.IdledAtAnnotation)
 		} else {
-			targetEndpoints.Annotations[unidlingapi.UnidleTargetAnnotation] = string(newAnnotationBytes)
+			targetService.Annotations[unidlingapi.UnidleTargetAnnotation] = string(newAnnotationBytes)
 		}
 	}
 
-	if _, err = c.endpointsNamespacer.Endpoints(info.Namespace).Update(context.TODO(), targetEndpoints, metav1.UpdateOptions{}); err != nil {
+	if _, err = c.servicesNamespacer.Services(info.Namespace).Update(context.TODO(), targetService, metav1.UpdateOptions{}); err != nil {
 		return true, fmt.Errorf("unable to update/remove idle annotations from %s/%s: %v", info.Namespace, info.Name, err)
 	}
 
+	// oc idle still annotates endpoints for backwards
+	// compatibilty. We need to remove any idled annotations on
+	// the endpoints.
+
+	// fetch the endpoints in question
+	targetEndpoints, err := c.endpointsNamespacer.Endpoints(info.Namespace).Get(context.TODO(), info.Name, metav1.GetOptions{})
+	if err != nil {
+		return true, fmt.Errorf("unable to retrieve endpoints: %v", err)
+	}
+
+	// make sure we actually were idled...
+	idledTimeRaw, wasIdled = targetEndpoints.Annotations[unidlingapi.IdledAtAnnotation]
+	if !wasIdled {
+		klog.V(5).Infof("UnidlingController received a NeedPods event for a service that was not idled, ignoring")
+		return false, nil
+	}
+
+	// ...and make sure this request was to wake up from the most recent idling, and not a previous one
+	idledTime, err = time.Parse(time.RFC3339, idledTimeRaw)
+	if err != nil {
+		// retrying here won't help, we're just stuck as idle since we can't get parse the idled time
+		return false, fmt.Errorf("unable to check idled-at time: %v", err)
+	}
+	if lastFired.Before(idledTime) {
+		klog.V(5).Infof("UnidlingController received an out-of-date NeedPods event, ignoring")
+		return false, nil
+	}
+
+	_, unidledAnnotation := targetEndpoints.Annotations[unidlingapi.UnidleTargetAnnotation]
+	_, idledAtAnnotation := targetEndpoints.Annotations[unidlingapi.IdledAtAnnotation]
+
+	if unidledAnnotation || idledAtAnnotation {
+		delete(targetEndpoints.Annotations, unidlingapi.UnidleTargetAnnotation)
+		delete(targetEndpoints.Annotations, unidlingapi.IdledAtAnnotation)
+
+		if _, err = c.endpointsNamespacer.Endpoints(info.Namespace).Update(context.TODO(), targetEndpoints, metav1.UpdateOptions{}); err != nil {
+			return true, fmt.Errorf("unable to update/remove idle annotations from %s/%s: %v", info.Namespace, info.Name, err)
+		}
+	}
+
 	return false, nil
 }
 
